#include <SoftwareSerial.h>

#include <TinyGPS.h>
#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>

#define ON_Board_LED 2  //--> Defining an On Board LED, used for indicators when the process of connecting to a wifi router

/* This sample code demonstrates the normal use of a TinyGPS object.
   It requires the use of SoftwareSerial, and assumes that you have a
   4800-baud serial GPS device hooked up on pins 4(rx) and 3(tx).
*/

TinyGPS gps;
SoftwareSerial ss(4, 2);

const char* ssid = "JioFiber-4G"; //--> Your wifi name or SSID.
const char* password = "1234567890"; //--> Your wifi password.


//----------------------------------------Host & httpsPort
const char* host = "script.google.com";
const int httpsPort = 443;
//----------------------------------------

WiFiClientSecure client; //--> Create a WiFiClientSecure object.
//https://script.google.com/macros/s/AKfycbxOsEA67mi8tfY9PfvSbbhz9qaHvoIhzlseHizzbiP7EaV1z7cc-gDdg59_42BeqNXzvw/exec?humidity=2
String GAS_ID = "AKfycbzawvSYbPRQgqNAP4fTMCIDY4yFh4E1oPMNkyYdVH18SXZ5Ii6O-YiC5IWwIO_Zkxps"; //--> spreadsheet script ID
//https://script.google.com/macros/s/AKfycbzawvSYbPRQgqNAP4fTMCIDY4yFh4E1oPMNkyYdVH18SXZ5Ii6O-YiC5IWwIO_Zkxps/exec
//https://script.google.com/macros/s/AKfycbzawvSYbPRQgqNAP4fTMCIDY4yFh4E1oPMNkyYdVH18SXZ5Ii6O-YiC5IWwIO_Zkxps/exec?destination=28.50124,77.09610
//https://script.google.com/macros/s/AKfycbzawvSYbPRQgqNAP4fTMCIDY4yFh4E1oPMNkyYdVH18SXZ5Ii6O-YiC5IWwIO_Zkxps/exec?location=28.50124,77.09610
//AKfycbzawvSYbPRQgqNAP4fTMCIDY4yFh4E1oPMNkyYdVH18SXZ5Ii6O-YiC5IWwIO_Zkxps
    float flat, flon;

void setup()
{
  Serial.begin(115200);
  ss.begin(4800);
  
  Serial.print("Simple TinyGPS library v. "); Serial.println(TinyGPS::library_version());
  Serial.println("by Mikal Hart");
  Serial.println();


  Serial.print("Connecting");
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    //----------------------------------------Make the On Board Flashing LED on the process of connecting to the wifi router.
    digitalWrite(ON_Board_LED, LOW);
    delay(250);
    digitalWrite(ON_Board_LED, HIGH);
    delay(250);
    //----------------------------------------
  }
  //----------------------------------------


  client.setInsecure();

}

void loop()
{
  bool newData = false;
  unsigned long chars;
  unsigned short sentences, failed;

  // For one second we parse GPS data and report some key values
  for (unsigned long start = millis(); millis() - start < 1000;)
  {
    while (ss.available())
    {
      char c = ss.read();
      // Serial.write(c); // uncomment this line if you want to see the GPS data flowing
      if (gps.encode(c)) // Did a new valid sentence come in?
        newData = true;
    }
  }

  if (newData)
  {
    //float flat, flon;
    unsigned long age;
    gps.f_get_position(&flat, &flon, &age);
    Serial.print("LAT=");
    Serial.print(flat == TinyGPS::GPS_INVALID_F_ANGLE ? 0.0 : flat, 6);
    Serial.print(" LON=");
    Serial.print(flon == TinyGPS::GPS_INVALID_F_ANGLE ? 0.0 : flon, 6);
    Serial.print(" SAT=");
    Serial.print(gps.satellites() == TinyGPS::GPS_INVALID_SATELLITES ? 0 : gps.satellites());
    Serial.print(" PREC=");
    Serial.print(gps.hdop() == TinyGPS::GPS_INVALID_HDOP ? 0 : gps.hdop());

    Serial.println("==========");
  Serial.print("connecting to ");
  Serial.println(host);
  
  //----------------------------------------Connect to Google host
  if (!client.connect(host, httpsPort)) {
    Serial.println("connection failed");
    return;
  }
  //----------------------------------------

  //----------------------------------------Processing data and sending data
  String string_location=  String(flat + "," + flon);
  // String string_temperature =  String(tem, DEC); 
  String string_destination =  String("28.50124,77.09610"; 
  String url = "/macros/s/" + GAS_ID + "/exec?location=" + string_location + "&destination=" + string_destination;
  Serial.print("requesting URL: ");
  Serial.println(url);

  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
         "Host: " + host + "\r\n" +
         "User-Agent: BuildFailureDetectorESP8266\r\n" +
         "Connection: close\r\n\r\n");

  Serial.println("request sent");
  //----------------------------------------

  //----------------------------------------Checking whether the data was sent successfully or not
  while (client.connected()) {
    String line = client.readStringUntil('\n');
    if (line == "\r") {
      Serial.println("headers received");
      break;
    }
  }
  String line = client.readStringUntil('\n');
  if (line.startsWith("{\"state\":\"success\"")) {
    Serial.println("esp8266/Arduino CI successfull!");
  } else {
    Serial.println("esp8266/Arduino CI has failed");
  }
  Serial.print("reply was : ");
  Serial.println(line);
  Serial.println("closing connection");
  Serial.println("==========");
  Serial.println();
  //---------------------------

  }
  
  gps.stats(&chars, &sentences, &failed);
  Serial.print(" CHARS=");
  Serial.print(chars);
  Serial.print(" SENTENCES=");
  Serial.print(sentences);
  Serial.print(" CSUM ERR=");
  Serial.println(failed);
  if (chars == 0)
    Serial.println("** No characters received from GPS: check wiring **");
}